@@ -49,7 +49,6 @@
 #include <Lum/Menu.h>
 #include <Lum/PercentBar.h>
 #include <Lum/PercentPie.h>
-#include <Lum/Space.h>
 #include <Lum/Table.h>
 #include <Lum/Text.h>
 #include <Lum/TextValue.h>
@@ -78,9 +77,8 @@
 
 static Lum::AppInfo info;
 
-class DiskInfo
+struct DiskInfo
 {
-public:
   std::wstring  name;
   std::wstring  dir;
   double        blocks;
@@ -264,12 +262,9 @@ std::wstring DoubleToWString(double value, double unit)
 Lum::TextValue* GetTextValueLeft(const Lum::Model::StringRef& value)
 {
   Lum::TextValue   *text;
-  Lum::OS::FontRef font(Lum::OS::display->GetFont(Lum::OS::Display::fontTypeProportional,
-                                                  Lum::OS::Display::fontScaleCaption1));
+  Lum::OS::FontRef font(Lum::OS::display->GetFont(Lum::OS::Display::fontScaleCaption1));
 
-  text=new Lum::TextValue();
-  text->SetFlex(true,false);
-  text->SetModel(value.Get());
+  text=Lum::TextValue::Create(value,Lum::TextValue::left,true,false);
   text->SetMinWidth(Lum::Base::Size::stdCharWidth,25);
   // TODO: font
 
@@ -279,13 +274,9 @@ Lum::TextValue* GetTextValueLeft(const Lum::Model::StringRef& value)
 Lum::TextValue* GetTextValueRight(const Lum::Model::StringRef& value)
 {
   Lum::TextValue   *text;
-  Lum::OS::FontRef font(Lum::OS::display->GetFont(Lum::OS::Display::fontTypeProportional,
-                                                  Lum::OS::Display::fontScaleCaption1));
+  Lum::OS::FontRef font(Lum::OS::display->GetFont(Lum::OS::Display::fontScaleCaption1));
 
-  text=new Lum::TextValue();
-  text->SetFlex(true,false);
-  text->SetModel(value.Get());
-  text->SetAlignment(Lum::TextValue::right);
+  text=Lum::TextValue::Create(value,Lum::TextValue::right,true,false);
   text->SetMinWidth(Lum::Base::Size::stdCharWidth,25);
   // TODO: font
 
@@ -499,8 +490,6 @@ public:
 
     fillFactor=lround((100*top->size)/size);
 
-    std::cout << fillFactor << std::endl;
-
     labels.clear();
     DrawEntry(draw,top,this->x+1,this->y+1,((width-2)*fillFactor)/100,height-2,1);
   }
@@ -513,7 +502,6 @@ public:
       if (x>=iter->x && y>=iter->y &&
           x<iter->x+(int)iter->width && y<iter->y+(int)iter->height) {
         label->Set(iter->label);
-        //std::cout << Lum::Base::WStringToString(iter->label) << std::endl;
         return;
       }
     }
@@ -559,13 +547,10 @@ public:
 
   void PreInit()
   {
-    Lum::ButtonRow *row;
     Lum::Panel     *vert;
-    Lum::TextValue *label;
     Map            *map;
 
-    vert=new Lum::VPanel();
-    vert->SetFlex(true,true);
+    vert=Lum::VPanel::Create(true,true);
     vert->SetWidth(Lum::Base::Size::workHRel,80);
     vert->SetHeight(Lum::Base::Size::workVRel,80);
 
@@ -573,19 +558,10 @@ public:
     map->SetFlex(true,true);
 
     vert->Add(map);
-
-    vert->Add(new Lum::VSpace());
-
-    label=new Lum::TextValue();
-    label->SetFlex(true,false);
-    label->SetModel(map->GetLabel());
-    vert->Add(label);
-
-    vert->Add(new Lum::VSpace());
-
-    row=Lum::ButtonRow::CreateClose(GetClosedAction());
-    row->SetFlex(true,false);
-    vert->Add(row);
+    vert->AddSpace();
+    vert->Add(Lum::TextValue::Create(map->GetLabel(),true,false));
+    vert->AddSpace();
+    vert->Add(Lum::ButtonRow::CreateClose(GetClosedAction(),true,false));
 
     SetMain(vert);
   }
@@ -637,7 +613,7 @@ public:
 
     todos.push_back(dir);
 
-    while (todos.size()>0) {
+    while (todos.size()>0 && !IsAborted()) {
       Lum::Base::Path              todo;
       Lum::Base::DirScanner::Entry entry;
       Lum::Base::Status            status;
@@ -707,10 +683,72 @@ public:
       }
     }
 
+    if (IsAborted()) {
+      delete top;
+      top=NULL;
+      todos.clear();
+
+      return;
+    }
+
     top->AccumulateSize();
   }
 };
 
+typedef Lum::Model::StdTable<DiskInfo>      DiskInfoModel;
+typedef Lum::Base::Reference<DiskInfoModel> DiskInfoModelRef;
+
+class DiskInfoDataProvider : public DiskInfoModel::DataProvider
+{
+private:
+  Lum::PercentBar                 *percent;
+  Lum::Model::DoubleDataStreamRef data;
+
+public:
+  DiskInfoDataProvider()
+  : percent(new Lum::PercentBar()),
+    data(new Lum::Model::DoubleDataStream())
+  {
+    percent->SetFlex(true,true);
+    percent->SetMinWidth(Lum::Base::Size::stdCharWidth,15);
+    percent->SetModel(data);
+
+    data->SetNotificationMode(Lum::Model::DoubleDataStream::notifyExplicit);
+    data->SetChannels(3);
+  }
+
+  virtual ~DiskInfoDataProvider()
+  {
+    delete percent;
+  }
+
+  std::wstring GetString(const DiskInfoModel::Iterator& iter, size_t column) const
+  {
+    switch (column) {
+    case 1:
+      return iter->name;
+    case 2:
+      return iter->dir;
+    default:
+      return L"";
+    }
+  }
+
+  Lum::Object* GetObject(const DiskInfoModel::Iterator& iter, size_t column) const
+  {
+    switch (column) {
+    case 3:
+      data->Set(0,(iter->blocks-iter->free)/iter->blocks);
+      data->Set(1,iter->avail/iter->blocks);
+      data->Set(2,(iter->free-iter->avail)/iter->blocks);
+
+      return percent;
+    default:
+      return NULL;
+    }
+  }
+};
+
 class MainWindow : public Lum::Dialog
 {
 private:
@@ -724,7 +762,7 @@ private:
 
   Lum::Table                         *table;
   Lum::Model::SingleLineSelectionRef selection;
-  Lum::Model::ListTableRef           tableModel;
+  DiskInfoModelRef                   tableModel;
   Lum::Model::StringRef              device;
   Lum::Model::StringRef              dir;
   Lum::Model::StringRef              used;
@@ -744,7 +782,7 @@ public:
     refreshTime(new Lum::Model::Int(10)),
     timer(new Lum::Model::Action()),
     selection(new Lum::Model::SingleLineSelection),
-    tableModel(new Lum::Model::ListTable(3)),
+    tableModel(new DiskInfoModel(new DiskInfoDataProvider())),
     device(new Lum::Model::String()),
     dir(new Lum::Model::String()),
     used(new Lum::Model::String()),
@@ -765,46 +803,40 @@ public:
     Observe(timer);
     Observe(GetOpenedAction());
 
+    Observe(scanTask.GetAbortedAction());
     Observe(scanTask.GetFinishedAction());
   }
 
   void PreInit()
   {
-    Lum::Label            *label;
     Lum::Panel            *panel,*panel2;
     Lum::PercentPie       *percent;
     Lum::Model::HeaderRef headerModel;
 
-    panel=new Lum::VPanel();
-    panel->SetFlex(true,true);
-    panel->SetWidth(Lum::Base::Size::workHRel,40);
+    panel=Lum::VPanel::Create(true,true);
+    panel->SetWidth(Lum::Base::Size::workHRel,20);
     panel->SetHeight(Lum::Base::Size::workVRel,40);
 
-    panel2=new Lum::HPanel();
-    panel2->SetFlex(true,false);
-
-    label=new Lum::Label();
-    label->AddLabel(L"Device:",GetTextValueLeft(device));
-    label->AddLabel(L"Directory:",GetTextValueLeft(dir));
-    label->AddLabel(L"Size:",GetTextValueRight(total));
-    label->AddLabel(L"Used:",GetTextValueRight(used));
-    label->AddLabel(L"Free:",GetTextValueRight(free));
-
-    panel2->Add(label);
-
-    panel2->Add(new Lum::HSpace(true));
+    panel2=Lum::HPanel::Create(true,false);
+    panel2->Add(Lum::Label::Create()
+                ->AddLabel(L"Device:",GetTextValueLeft(device))
+                ->AddLabel(L"Directory:",GetTextValueLeft(dir))
+                ->AddLabel(L"Size:",GetTextValueRight(total))
+                ->AddLabel(L"Used:",GetTextValueRight(used))
+                ->AddLabel(L"Free:",GetTextValueRight(free)));
+    panel2->AddSpace(true);
 
     percent=new Lum::PercentPie();
     percent->SetModel(data);
     panel2->Add(percent);
 
     panel->Add(panel2);
-    panel->Add(new Lum::VSpace());
+    panel->AddSpace();
 
     headerModel=new Lum::Model::HeaderImpl();
     headerModel->AddColumn(L"Dir",Lum::Base::Size::stdCharWidth,10);
     headerModel->AddColumn(L"Dev",Lum::Base::Size::stdCharWidth,10);
-    headerModel->AddColumn(L"Size",Lum::Base::Size::stdCharWidth,20,true);
+    headerModel->AddColumn(L"Size",Lum::Base::Size::stdCharWidth,15,true);
 
     table=new Lum::Table();
     table->SetFlex(true,true);
@@ -847,48 +879,17 @@ public:
         size_t i;
 
         for (i=1; i<=tableModel->GetRows(); i++) {
-          if (iter->dir==tableModel->GetEntry(i)->GetString(1)) {
+          if (iter->dir==tableModel->GetEntry(i).dir) {
             break;
           }
         }
 
         if (i<=tableModel->GetRows()) {
-          Lum::Model::ListTable::Entry    *entry=tableModel->GetEntry(i);
-
-          Lum::PercentBar                 *percent;
-          Lum::Model::DoubleDataStreamRef data;
-
-          percent=dynamic_cast<Lum::PercentBar*>(entry->GetObject(3));
-          data=dynamic_cast<Lum::Model::DoubleDataStream*>(percent->GetModel());
-
-          data->Set(0,(iter->blocks-iter->free)/iter->blocks);
-          data->Set(1,iter->avail/iter->blocks);
-          data->Set(2,(iter->free-iter->avail)/iter->blocks);
-          data->Notify();
+          tableModel->GetEntry(i)=*iter;
+          tableModel->RedrawRow(i);
         }
         else {
-          Lum::PercentBar                 *percent;
-          Lum::Model::DoubleDataStreamRef data;
-          Lum::Model::ListTable::StdEntry *entry;
-
-          percent=new Lum::PercentBar();
-          percent->SetFlex(true,true);
-          percent->SetMinWidth(Lum::Base::Size::stdCharWidth,30);
-
-          data=new Lum::Model::DoubleDataStream();
-          data->SetNotificationMode(Lum::Model::DoubleDataStream::notifyExplicit);
-          data->SetChannels(3);
-          data->Set(0,(iter->blocks-iter->free)/iter->blocks);
-          data->Set(1,iter->avail/iter->blocks);
-          data->Set(2,(iter->free-iter->avail)/iter->blocks);
-          percent->SetModel(data);
-
-          entry=new Lum::Model::ListTable::StdEntry(tableModel);
-          entry->SetString(1,iter->dir);
-          entry->SetString(2,iter->name);
-          entry->SetObject(3,percent);
-
-          tableModel->Append(entry);
+          tableModel->Append(*iter);
         }
 
         ++iter;
@@ -899,7 +900,7 @@ public:
       size_t i=1;
 
       while(i<=tableModel->GetRows()) {
-        std::wstring                        directory=tableModel->GetEntry(i)->GetString(1);
+        std::wstring                        directory=tableModel->GetEntry(i).dir;
         std::list<DiskInfo>::const_iterator iter;
 
         iter=list.begin();
@@ -927,10 +928,8 @@ public:
     std::list<DiskInfo>::const_iterator iter;
 
     if (selection->HasSelection()) {
-      Lum::Model::ListTable::Entry *entry=tableModel->GetEntry(selection->GetLine());
-
       iter=list.begin();
-      while (iter!=list.end() && iter->dir!=entry->GetString(1)) {
+      while (iter!=list.end() && iter->dir!=tableModel->GetEntry(selection->GetLine()).dir) {
         ++iter;
       }
     }
@@ -969,10 +968,9 @@ public:
       return;
     }
 
-    Lum::Model::ListTable::Entry       *entry=tableModel->GetEntry(selection->GetLine());
     std::list<DiskInfo>::const_iterator iter=list.begin();
 
-    while (iter!=list.end() && iter->dir!=entry->GetString(1)) {
+    while (iter!=list.end() && iter->dir!=tableModel->GetEntry(selection->GetLine()).dir) {
       ++iter;
     }
 
